@@ -540,7 +540,7 @@ GetBaseOptions (const char *filename)
 {
     START_TIME(started);
 
-	ReloadASEnvironment( NULL, NULL, NULL, False );
+	ReloadASEnvironment( NULL, NULL, NULL, False, True );
 
     SHOW_TIME("BaseConfigParsingTime",started);
 }
@@ -770,6 +770,13 @@ DispatchEvent (ASEvent * event)
 				update_wharf_folder_styles( WharfState.root_folder, True );
 			}
             break;
+		default:
+#ifdef XSHMIMAGE
+			LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : completion_type = %d, event->type = %d ", Scr.ShmCompletionEventType, event->x.type );
+			if( event->x.type == Scr.ShmCompletionEventType )
+				handle_ShmCompletion( event );
+#endif /* SHAPE */
+			break;
     }
 }
 
@@ -2103,6 +2110,7 @@ check_swallow_window( ASWindowData *wd )
 	Bool withdraw_btn ;
     ASCanvas *nc ;
     int swidth, sheight ;
+	Window icon_w = None ;
 
     if( wd == NULL && !get_flags( wd->state_flags, AS_Mapped))
         return;
@@ -2161,22 +2169,43 @@ check_swallow_window( ASWindowData *wd )
     sleep_a_millisec(100);
 	grab_swallowed_canvas_btns( nc, aswb, withdraw_btn );
 
-    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
-		wd->icon != None )
+    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) )
+		icon_w = wd->icon ;
+	else if( get_flags( wd->flags, AS_WMDockApp ) ) 
+	{
+		XWMHints *wm_hints ;
+		if ((wm_hints = XGetWMHints (dpy, wd->client)) != NULL)
+		{
+			if( get_flags (wm_hints->flags, IconWindowHint) )
+				icon_w = wm_hints->icon_window ;
+			XFree (wm_hints);
+		}	
+	}
+
+	if( icon_w != None ) 
     {
-        ASCanvas *ic = create_ascanvas_container( wd->icon );
+        ASCanvas *ic = create_ascanvas_container( icon_w );
         aswb->swallowed->iconic = ic;
-        XReparentWindow( dpy, wd->icon, aswb->canvas->w, (aswb->canvas->width-ic->width)/2, (aswb->canvas->height-ic->height)/2 );
-        register_object( wd->icon, (ASMagic*)aswb );
-        XSelectInput (dpy, wd->icon, StructureNotifyMask);
+        XReparentWindow( dpy, icon_w, aswb->canvas->w, (aswb->canvas->width-ic->width)/2, (aswb->canvas->height-ic->height)/2 );
+        register_object( icon_w, (ASMagic*)aswb );
+        XSelectInput (dpy, icon_w, StructureNotifyMask);
 		ASSync(False);
 		grab_swallowed_canvas_btns( ic, aswb, withdraw_btn );
     }
-    aswb->swallowed->current = ( get_flags( wd->state_flags, AS_Iconic ) &&
-                                    aswb->swallowed->iconic != NULL )?
-                                aswb->swallowed->iconic:aswb->swallowed->normal;
+    aswb->swallowed->current = aswb->swallowed->normal;
+	if( aswb->swallowed->iconic != NULL )
+	{	
+		if( get_flags( wd->flags, AS_WMDockApp ) ||
+#if 0
+			(wd->res_class && strcasecmp(wd->res_class, "DockApp") == 0) || 
+#endif				
+			get_flags( wd->state_flags, AS_Iconic ) )
+		{
+			aswb->swallowed->current = aswb->swallowed->iconic ;
+		}
+	}
     handle_canvas_config( aswb->swallowed->current );
-    LOCAL_DEBUG_OUT( "client(%lX)->icon(%lX)->current(%lX)", wd->client, wd->icon, aswb->swallowed->current->w );
+    LOCAL_DEBUG_OUT( "client(%lX)->icon(%lX)->current(%lX)", wd->client, icon_w, aswb->swallowed->current->w );
 
     if( get_flags( aswb->flags, ASW_MaxSwallow ) ||
 		(Config->force_size.width == 0 && !get_flags(aswb->flags, ASW_FixedWidth)))
@@ -2184,8 +2213,23 @@ check_swallow_window( ASWindowData *wd )
     if( get_flags( aswb->flags, ASW_MaxSwallow ) ||
 		(Config->force_size.height == 0 && !get_flags(aswb->flags, ASW_FixedHeight)) )
         aswb->desired_height = aswb->swallowed->current->height;
-    swidth = min( aswb->desired_width, aswb->swallowed->current->width );
-    sheight = min( aswb->desired_height, aswb->swallowed->current->height );
+
+    swidth = aswb->swallowed->current->width ;
+	if( get_flags( wd->flags, AS_MinSize ) && swidth < wd->hints.min_width)
+   		swidth = wd->hints.min_width;
+	if( swidth <= 2 ) 
+   		swidth = aswb->desired_width;
+	else if( swidth > aswb->desired_width ) 
+		swidth = aswb->desired_width ;
+    
+    sheight = aswb->swallowed->current->height ;
+	if( get_flags( wd->flags, AS_MinSize ) && sheight < wd->hints.min_height)
+   		sheight = wd->hints.min_height;
+	if( sheight <= 2 ) 
+   		sheight = aswb->desired_height;
+	else if( sheight > aswb->desired_height ) 
+		sheight = aswb->desired_height ;
+
     moveresize_canvas( aswb->swallowed->current,
                        make_tile_pad( get_flags(Config->align_contents,PAD_LEFT),
                                       get_flags(Config->align_contents,PAD_RIGHT),
@@ -2194,6 +2238,22 @@ check_swallow_window( ASWindowData *wd )
                                       get_flags(Config->align_contents,PAD_BOTTOM),
                                       aswb->canvas->height, sheight    ),
                        swidth, sheight );
+	if( !get_flags( wd->flags, AS_ClientIcon ) ) 
+	{  /* workaround for broken wmdock apps that draw into icon, that is a child of a client, 
+		* and at the same time not properly sized ( Just don't ask - some ppl are wierd ) */
+		XWMHints *wm_hints = XGetWMHints (dpy, wd->client);
+		LOCAL_DEBUG_OUT( "wmfire workaround step1: wm_hints = %p", wm_hints );
+		if( wm_hints ) 
+		{
+			LOCAL_DEBUG_OUT( "wmfire workaround step2: icon_window_hint = %ld, icon_window = %lX", get_flags (wm_hints->flags, IconWindowHint), wm_hints->icon_window );
+			if( get_flags (wm_hints->flags, IconWindowHint) && wm_hints->icon_window != None )				   
+			{	
+				LOCAL_DEBUG_OUT( "wmfire workaround step3: resizing to %dx%d", swidth, sheight );
+				XResizeWindow( dpy, wm_hints->icon_window, swidth, sheight );
+			}
+			XFree (wm_hints);			   
+		}
+	}
     map_canvas_window( aswb->swallowed->current, True );
     send_swallowed_configure_notify(aswb);
 	update_wharf_folder_size( aswf );
