@@ -64,6 +64,7 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module );
 void close_func_handler( FunctionData *data, ASEvent *event, int module );
 void restart_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_func_handler( FunctionData *data, ASEvent *event, int module );
+void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_background_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_config_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_theme_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -86,6 +87,8 @@ void send_window_list_func_handler( FunctionData *data, ASEvent *event, int modu
 void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module );
 void test_func_handler( FunctionData *data, ASEvent *event, int module );
 void screenshot_func_handler( FunctionData *data, ASEvent *event, int module );
+void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module );
+
 
 /* handlers initialization function : */
 void SetupFunctionHandlers()
@@ -134,9 +137,10 @@ void SetupFunctionHandlers()
     function_handlers[F_RESTART]            = restart_func_handler ;
 
     function_handlers[F_EXEC] =
-	function_handlers[F_Swallow] =
-	function_handlers[F_MaxSwallow] =
-	function_handlers[F_DropExec]       = exec_func_handler ;
+		function_handlers[F_Swallow] =
+		function_handlers[F_MaxSwallow] 	= exec_func_handler ;
+
+	function_handlers[F_ExecInTerm]       	= exec_in_term_func_handler ;
 
     function_handlers[F_CHANGE_BACKGROUND]  = change_background_func_handler;
 
@@ -188,6 +192,7 @@ void SetupFunctionHandlers()
 	function_handlers[F_TAKE_WINDOWSHOT]    =
 	function_handlers[F_TAKE_FRAMESHOT]    =
 		function_handlers[F_TAKE_SCREENSHOT]    = screenshot_func_handler ;
+	function_handlers[F_SWALLOW_WINDOW]	= swallow_window_func_handler ;
 }
 
 /* complex functions are stored in hash table ComplexFunctions */
@@ -372,6 +377,7 @@ DoExecuteFunction ( ASScheduledFunction *sf )
     }
 
     /* Defer Execution may wish to alter this value */
+	LOCAL_DEBUG_OUT( "client = %p, IsWindowFunc() = %s", event->client, IsWindowFunc (func)?"Yes":"No" );
     if (IsWindowFunc (func))
 	{
 		int           do_defer = !(sf->defered), fin_event;
@@ -383,7 +389,8 @@ DoExecuteFunction ( ASScheduledFunction *sf )
 		else
 	    	fin_event = ButtonPress ;
 
-		if (data->text != NULL && event->client == NULL)
+		if (data->text != NULL && event->client == NULL && 
+			func != F_SWALLOW_WINDOW )/* SWallowWindow has module name as its text ! */
 			if (*(data->text) != '\0')
 				if ((event->client = pattern2ASWindow (data->text)) != NULL)
 				{
@@ -472,25 +479,28 @@ LOCAL_DEBUG_CALLER_OUT( "cursor %d, event %d, window 0x%lX, window_name \"%s\",
 */
     if (!(res = !GrabEm (ASDefaultScr, Scr.Feel.cursors[cursor])))
 	{
-	WaitEventLoop( event, finish_event, -1 );
-	LOCAL_DEBUG_OUT( "window(%lX)->root(%lX)->subwindow(%lX)", event->x.xbutton.window, event->x.xbutton.root, event->x.xbutton.subwindow );
-	if (event->client == NULL)
-	{
-	    res = True ;
-	    /* since we grabbed cursor we may get clicks over client windows as reported
-	     * relative to the root window, in which case we have to check subwindow to
-	     * see what client was clicked */
-	    if( event->x.xbutton.subwindow != event->w )
-	    {
-		event->client = window2ASWindow( event->x.xbutton.subwindow );
-		if( event->client != NULL )
+		ASHintWindow *hint = create_ashint_window( ASDefaultScr,  &(Scr.Look), "Please select target window");
+   		WaitEventLoop( event, finish_event, -1, hint );
+		destroy_ashint_window( &hint );
+
+		LOCAL_DEBUG_OUT( "window(%lX)->root(%lX)->subwindow(%lX)", event->x.xbutton.window, event->x.xbutton.root, event->x.xbutton.subwindow );
+		if (event->client == NULL)
 		{
-		    res = False ;
-		    event->w = event->x.xbutton.subwindow ;
+	    	res = True ;
+	    	/* since we grabbed cursor we may get clicks over client windows as reported
+	     	* relative to the root window, in which case we have to check subwindow to
+	     	* see what client was clicked */
+	    	if( event->x.xbutton.subwindow != event->w )
+	    	{
+				event->client = window2ASWindow( event->x.xbutton.subwindow );
+				if( event->client != NULL )
+				{
+		    		res = False ;
+		    		event->w = event->x.xbutton.subwindow ;
+				}
+	    	}
 		}
-	    }
-	}
-	UngrabEm ();
+		UngrabEm ();
     }
     if( res )
 	XBell (dpy, event->scr->screen);
@@ -671,6 +681,7 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 	int height = asw->status->height ;
 
 	new_val1 = APPLY_VALUE_UNIT(Scr.MyDisplayWidth,data->func_val[0],data->unit_val[0]);
+	LOCAL_DEBUG_OUT( "val1 = %ld, unit1 = %ld, new_val1 = %d", data->func_val[0],data->unit_val[0], new_val1 );
 	new_val2 = APPLY_VALUE_UNIT(Scr.MyDisplayHeight,data->func_val[1],data->unit_val[1]);
 	if( data->func == F_MOVE )
 	{
@@ -886,7 +897,7 @@ void setlayer_func_handler( FunctionData *data, ASEvent *event, int module )
 void change_desk_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     if( event->client )
-	change_aswindow_desktop( event->client, data->func_val[0] );
+	change_aswindow_desktop( event->client, data->func_val[0], False );
 }
 
 void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -902,8 +913,8 @@ void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module
 			toggle_flags |= AS_MaximizedY ;
 		if( event->client )
 		{
-			event->client->maximize_ratio_x =  (data->func_val[0] * data->unit_val[0]) / Scr.MyDisplayWidth ;
-			event->client->maximize_ratio_y =  (data->func_val[1] * data->unit_val[1]) / Scr.MyDisplayHeight ;
+			event->client->maximize_ratio_x = (data->unit_val[0] == 0 )? data->func_val[0]: (data->func_val[0]*data->unit_val[0]*100)/Scr.MyDisplayWidth ;
+			event->client->maximize_ratio_y = (data->unit_val[1] == 0 )? data->func_val[1]: (data->func_val[1]*data->unit_val[1]*100)/Scr.MyDisplayHeight ;
 		}
     }else if( data->func == F_SHADE )
 	toggle_flags = AS_Shaded ;
@@ -956,18 +967,19 @@ void bookmark_window_func_handler( FunctionData *data, ASEvent *event, int modul
 void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     ASMenu *menu = NULL;
-    if( data->name )
-	menu = find_asmenu( data->name );
+	char * menu_name = data->text? data->text:data->name ;
+    if( menu_name )
+		menu = find_asmenu( menu_name );
     else if( event->client && event->client->internal )
     {
-	ASMagic *data = event->client->internal->data ;
-	if( data->magic == MAGIC_ASMENU )
-	    menu = (ASMenu*)data;
+		ASMagic *data = event->client->internal->data ;
+		if( data->magic == MAGIC_ASMENU )
+	    	menu = (ASMenu*)data;
     }
     if( menu == NULL )
-	XBell (dpy, event->scr->screen);
+		XBell (dpy, event->scr->screen);
     else
-	pin_asmenu( menu );
+		pin_asmenu( menu );
 }
 
 
@@ -1012,6 +1024,123 @@ void exec_func_handler( FunctionData *data, ASEvent *event, int module )
     XSync (dpy, 0);
 }
 
+int find_escaped_chr_pos( const char *str, char c )
+{
+	int i ; 
+	for( i = 0 ; str[i] != '\0' ; ++i ) 
+	{
+		if( str[i] == '\\' ) 
+		{	
+			if( str[++i] == '\0' ) 
+				break;
+		}else if( str[i] == c ) 
+			break;
+	}		   
+	return i;	
+}	 
+
+char *parse_term_cmdl( const char *term_name, const char *term_command, const char *cmdl )
+{
+	int term_name_len, cmdl_len, curr_full, curr_cmdl;
+	char *full_cmdl = NULL ; 
+	Bool first = True ;
+	
+	if( term_name == NULL || term_command == NULL || cmdl == NULL ) 
+		return NULL;
+
+	LOCAL_DEBUG_OUT( "term_name = \"%s\", term_command = \"%s\", cmdl = \"%s\"", 
+					 term_name, term_command, cmdl );
+	curr_full = strlen(term_command);
+	cmdl_len  = strlen(cmdl);
+	term_name_len = strlen(term_name);
+	curr_cmdl = 0 ;
+	full_cmdl = safemalloc( curr_full+4+cmdl_len+1 );
+	
+	strcpy( full_cmdl, term_command );
+
+	while( curr_cmdl < cmdl_len ) 
+	{
+		while( isspace(cmdl[curr_cmdl])) ++curr_cmdl;
+		if( mystrncasecmp( &(cmdl[curr_cmdl]), "if(", 3 ) == 0 ) 
+		{
+			int tmp ;
+
+			curr_cmdl += 3 ;
+			tmp = curr_cmdl ; 
+			curr_cmdl += find_escaped_chr_pos( &(cmdl[curr_cmdl]), '}' )+1;
+			while( isspace(cmdl[tmp]) ) ++tmp ;
+			if( mystrncasecmp( &(cmdl[tmp]), term_name, term_name_len ) == 0 )
+			{
+				tmp += term_name_len ;	  
+				while( isspace(cmdl[tmp]) ) ++tmp ;
+				if( cmdl[tmp] == ')' )
+				{
+					++tmp ;
+					while( isspace(cmdl[tmp]) ) ++tmp ;
+					if( cmdl[tmp] == '{' ) 
+						++tmp ;
+					while( isspace(cmdl[tmp]) ) ++tmp ;
+					full_cmdl[curr_full++] = ' ';
+					while( tmp < curr_cmdl-1 ) 
+						full_cmdl[curr_full++] = cmdl[tmp++];
+				}
+			}
+		}else	 
+		{
+			if( first )
+			{	
+				if( cmdl[curr_cmdl] != '-' ) 	
+				{
+					if( cmdl[curr_cmdl] != '\0' ) 	  
+						sprintf( &(full_cmdl[curr_full]), " -e %s", &(cmdl[curr_cmdl]));
+					else
+						full_cmdl[curr_full] = '\0' ;
+					return full_cmdl;
+				}	 
+				first = False ;
+			}
+			
+			if( strncmp( &(cmdl[curr_cmdl]), "-e ", 3 ) == 0 ) 
+			{
+				sprintf( &(full_cmdl[curr_full]), " %s", &(cmdl[curr_cmdl]));
+				return full_cmdl;
+			}	 
+			
+			full_cmdl[curr_full++] = ' ';
+			while( curr_cmdl < cmdl_len && !isspace(cmdl[curr_cmdl])) 
+				full_cmdl[curr_full++] = cmdl[curr_cmdl++];
+		}	 
+		
+	}	 
+	full_cmdl[curr_full] = '\0' ;
+	
+	return full_cmdl;	
+}	 
+
+void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	if( Environment->term_command != NULL && data->text != NULL ) 
+	{
+		char *full_cmdl = NULL;
+		char *term_name = strrchr( Environment->term_command, '/' );
+		term_name = (term_name == NULL )? Environment->term_command: term_name+1 ;
+		full_cmdl = parse_term_cmdl( term_name, Environment->term_command, data->text );
+		if( full_cmdl ) 
+		{	
+			LOCAL_DEBUG_OUT( "full_cmdl = [%s]", full_cmdl );
+    		XGrabPointer( dpy, Scr.Root, True,
+			      		ButtonPressMask | ButtonReleaseMask,
+		  		GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
+    		XSync (dpy, 0);
+    		spawn_child( full_cmdl, -1, -1, None, C_NO_CONTEXT, True, False, NULL );
+    		XUngrabPointer (dpy, CurrentTime);
+			free( full_cmdl );
+		}
+    	XSync (dpy, 0);
+	}
+}
+
+
 static int _as_config_change_recursion = 0 ;
 static int _as_config_change_count = 0 ;
 static int _as_background_change_count = 0 ;
@@ -1313,7 +1442,6 @@ void desk_func_handler( FunctionData *data, ASEvent *event, int module )
 		new_desk = data->func_val[1];
 	else 
 		return ;
-
     ChangeDesks (new_desk);
 }
 
@@ -1357,7 +1485,7 @@ void killmodule_func_handler( FunctionData *data, ASEvent *event, int module )
 
 void popup_func_handler( FunctionData *data, ASEvent *event, int module )
 {
-    run_menu( data->name, event->client?event->client->w:None );
+    run_menu( data->text?data->text:data->name, event->client?event->client->w:None );
 }
 
 void quit_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1388,10 +1516,11 @@ send_aswindow_data_iter_func(void *data, void *aux_data)
 	module_id.ptr = aux_data ;
 
     SendConfig (module_id.id, M_CONFIGURE_WINDOW, asw);
-    SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], asw->hints->names_encoding[0]);
-    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, asw->hints->names_encoding[asw->hints->icon_name_idx]);
+	/* always start with RES_CLASS to let module know if this is a DockApp or not */
     SendString (module_id.id, M_RES_CLASS, asw->w, asw->frame, asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
     SendString (module_id.id, M_RES_NAME,  asw->w, asw->frame, asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
+    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, asw->hints->names_encoding[asw->hints->icon_name_idx]);
+    SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], asw->hints->names_encoding[0]);
     return True;
 }
 
@@ -1414,7 +1543,7 @@ void screenshot_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	ASImage *im ;
 	Window target = None ;
-	sleep_a_millisec(500);
+	sleep_a_millisec(300);
 	if( event->client && data->func != F_TAKE_SCREENSHOT)
 		target = (data->func == F_TAKE_WINDOWSHOT)?event->client->w:event->client->frame;
 	im = grab_root_asimage( ASDefaultScr, target, True );
@@ -1441,22 +1570,16 @@ void screenshot_func_handler( FunctionData *data, ASEvent *event, int module )
 		}
 		if( realfilename == NULL )
 		{
-			char capture_file_name[80];
+			char *capture_file_name = DEFAULT_CAPTURE_SCREEN_FILE;
+			char *default_template;
 			if (data->func == F_TAKE_WINDOWSHOT)
-			{
-			    strcpy(capture_file_name, DEFAULT_CAPTURE_WINDOW_FILE);
-			}
+			    capture_file_name = DEFAULT_CAPTURE_WINDOW_FILE;
 			else if (data->func == F_TAKE_FRAMESHOT)
-			{
-			    strcpy(capture_file_name, DEFAULT_CAPTURE_FRAMEDWINDOW_FILE);
-			}
-			else if (data->func == F_TAKE_SCREENSHOT)
-			{
-			    strcpy(capture_file_name, DEFAULT_CAPTURE_SCREEN_FILE);
-			}
-			char default_template[128];
-			sprintf(&(default_template[0]), strcat(capture_file_name, ".%lu.png"), time(NULL));
-			realfilename = PutHome(&(default_template[0]));
+			    capture_file_name = DEFAULT_CAPTURE_FRAMEDWINDOW_FILE;
+			default_template = safemalloc(strlen(capture_file_name)+100);
+			sprintf(default_template, "%s.%lu.png", capture_file_name, time(NULL));
+			realfilename = PutHome(default_template);
+			free( default_template );
 			compress = "9" ;
 			type = "png" ;
 		}
@@ -1467,6 +1590,18 @@ void screenshot_func_handler( FunctionData *data, ASEvent *event, int module )
 	}
 }
 
+void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	if( event->client ) 
+	{	
+		if( data->text ) 
+			
+			
+			
+			module = FindModuleByName( data->text );
+		SendPacket( module, M_SWALLOW_WINDOW, 2, event->client->w, event->client->frame);
+ 	}	
+}	 
 
 void test_func_handler( FunctionData *data, ASEvent *event, int module )
 {
